/* ---------------------------------------------------------- */
/*  pako_simple_app.cpp                                       */
/* ---------------------------------------------------------- */

/*------------------------------------------------------------- */
/** @file
@brief    Pako SDL Game Library - Foundamental SDL application class
@author   rerofumi
@since    Nov.04.2006
*/
/*-----------------------------------------------------
Copyright (C) 2006,2011 fumi2kick/rerofumi <rero2@yuumu.org>
All Rights Reserved.
------------------------------------------------------
Update:
Apr.07.2011 - marge input/sound module
------------------------------------------------------*/

/*-------------------------------*/
/* include                       */
/*-------------------------------*/

#include <vector>
#include <fstream>

#include "pako_simple_app.h"

/*-------------------------------*/
/* define                        */
/*-------------------------------*/

//! fobO\͍͒`
#define DEBUGPRINT(FMT, ...)

const static std::string SOUND_LIST_FILE = "sounddat.lst";

/*-------------------------------*/
/* global value                  */
/*-------------------------------*/

/*-------------------------------*/
/* local routine                 */
/*-------------------------------*/

/*-------------------------------*/
/* implementatiion               */
/*-------------------------------*/

/*---------------------------------------------------------------*/
/*                                                               */
/* Application base class                                        */
/*                                                               */
/*---------------------------------------------------------------*/

/*------------------------------------------------------*/
/* constructor/destructor                               */
/*------------------------------------------------------*/

/**
* RXgN^
*/
PakoSimpleApp::PakoSimpleApp()
    :m_TargetMachine(0)
    ,m_WindowSizeW(PAKO_FRAME_SIZE_W)
    ,m_WindowSizeH(PAKO_FRAME_SIZE_H)
    ,m_SdlVideoModeFlag(0)
    ,m_FrameRate(PAKO_FRAME_RATE)
    ,m_FrameSkipMode(1)
    ,m_SkipRate(0)
    ,m_AppCounter(0)
    ,m_pInput(0)
    ,m_pSound(0)
    ,m_bFullScreen(false)
	,m_iSoundParamFreq(44100)
	,m_iSoundParamBuff(1024)
{
}

/**
* fXgN^(n)
*/
PakoSimpleApp::~PakoSimpleApp()
{
}


/*------------------------------------------------------*/
/* interface                                            */
/*------------------------------------------------------*/

/* ----------------------------------------- */
/* --- execute() */
/**
* Q[{̂s郁\bh
* ̃\bh̒Ŏs[vWJAQ[I܂
* \bh𔲂Ȃ
* {I[o[Ch͍lȂ
*
* @return IR(\)
*
*/
int PakoSimpleApp::execute(void)
{
    int  i;
    int  rs;
    int endflag;
    bool endflag2;
    /* - ^CJEg */
    int  SystemTime;
    int  BeforeTiming;
    int  WorkTime;
    int  NowTiming;
    int  DispTime;

    /* --- O */
    rs = onBoot();
    /* - Ȃɂ珉Ɏs */
    if (rs != 0) {
        return(1);
    }
    /* --- AvP[VI */
    rs = onExecuteBefore();
    /* - Ȃɂ珉Ɏs */
    if (rs != 0) {
        return(1);
    }

    /* --- ^C~Ov */
    WorkTime = 0;
    SystemTime = SDL_GetTicks();
    BeforeTiming = SystemTime;
    DispTime = 0;
    m_AppCounter = 0;
    m_SkipRate = 0;
    /* --- ItO */
    endflag = 0;
    endflag2 = true;

    /* ---------------------------------------- */
    /* ----- C[`                     */
    /* ---------------------------------------- */
    while(endflag == 0) {
        /* --- Q[{̃[v */
        for(i=0; i<=m_SkipRate; i++) {
            m_AppCounter = m_AppCounter + 1;
            m_pInput->poll();
            /* ------------------------------- */
            /* ----- Q[C */
            if (((endflag2 = tick()) == false) || (m_pInput->isExit() == true)) {
                BeforeTiming = SDL_GetTicks();
                break;
            }
        }

        /* ----- fobOpĎ */
        DEBUGPRINT("%d", m_AppCounter);
        DEBUGPRINT("Work Time : W%02d/D%02d/%d", WorkTime, DispTime, (1000/m_FrameRate));
        DEBUGPRINT("Frame Skip : %d", FrameSkip);
        DEBUGPRINT("Key Input : %x", Inputm_JoyKey(0));

        WorkTime = SDL_GetTicks() - BeforeTiming;

		/* --- t[IAXN[` */
        if (tickDisplay() == false) {
            break;
        }
        swapFrameBuffer();

        /* --- t[^C}[ */
        DispTime = SDL_GetTicks() - BeforeTiming;
		m_SystemTime = DispTime;
		NowTiming = (1000 / m_FrameRate) - m_SystemTime;
        if ((NowTiming > 0) && (NowTiming <= (1000 / m_FrameRate))) {
            SDL_Delay(NowTiming);
        }
        /* --- IĎ */
        endflag = (m_pInput->isExit() == true);
        if (endflag2 == false) {
            endflag = 1;
        }
        /* --- t[XLbv[gvZ */
        if (m_FrameSkipMode == PAKO_FRAME_SKIP_DISPLAY) {
            m_SkipRate = DispTime / (1000 / m_FrameRate);
        }
        else {
            m_SkipRate = 0;
        }
        m_BeforeTiming = BeforeTiming = SDL_GetTicks();

#if 0
		/* --- F11 ŃtXN[[h */
        if (m_pInput->isFull() == true) {
            m_bFullScreen = !m_bFullScreen;
            onExecuteAfter();
            setFullScreen(m_bFullScreen);
            onExecuteBefore();
        }
#endif

        /* ----- C[vRR܂ */
    }  

    /* --- ㏈ */
    rs = onExecuteAfter();
    rs = onShutdown();
    /* --- ܂ */
    return(rs);
}


/* ----------------------------------------- */
/* --- onBoot() */
/**
* execute ̓ŌĂяoO\bh
* VXȅɎg
* pŏڂǉĂ
*
* @return 0ŒʏIAȊOŃG[vʒm
*
*/
int PakoSimpleApp::onBoot(void)
{
    int  r;

    /* ----- SDL  */
    r = SDL_Init(SDL_INIT_VIDEO |
        SDL_INIT_AUDIO | 
        SDL_INIT_TIMER |
        SDL_INIT_JOYSTICK );
    if (r != 0) {
        /* --- SDLłȂ */
        return(1);
    }
	SDL_WM_SetCaption(m_strWindowTitle.c_str(), m_strWindowTitle.c_str());

    /* ----- SDLo͏ */
    m_pInput = new Input;
    m_pSound = new Sound(m_iSoundParamFreq, m_iSoundParamBuff);

    /* - - - ʕ\̏͌pɂ */

    return(0);
}


/* ----------------------------------------- */
/* --- onShutdown() */
/**
* execute ̏IĂяo㏈\bh
* VXe̔j
*
* @return 0ŒʏIAȊOŃG[vʒm
*
*/
int PakoSimpleApp::onShutdown(void)
{
    /* --- IASDL ㏈ */
    delete m_pSound;
    m_pSound = 0;
    delete m_pInput;
    m_pInput = 0;
    SDL_Quit();

    return(0);
}



/* ----------------------------------------- */
/* --- onExecuteBefore() */
/**
* execute ̓ŌĂяoO\bh
* Q[NX̏Ɏg
* pŏڂǉĂ
*
* @return 0ŒʏIAȊOŃG[vʒm
*
*/
int PakoSimpleApp::onExecuteBefore(void)
{
    return(0);
}


/* ----------------------------------------- */
/* --- onExecuteBefore() */
/**
* execute ̏IĂяo㏈\bh
* jڂȂp̂ɒǉ
*
* @return 0ŒʏIAȊOŃG[vʒm
*
*/
int PakoSimpleApp::onExecuteAfter(void)
{
    return(0);
}


/* ----------------------------------------- */
/* --- setFrameRate() */
/**
* t[[gl fps Ŏw肷ANZT
*
* @param fps [in] t[[g̒l(l60)
*
*/
void PakoSimpleApp::setFrameRate(int fps)
{
    m_FrameRate = fps;
}

/* ----------------------------------------- */
/* --- setScreenSize() */
/**
* t[TCYݒ肷ANZT
*
* @param width  [in] ʃTCY
* @param height [in] ʃTCYc
*/
void PakoSimpleApp::setScreenSize(int width, int height)
{
    m_WindowSizeW = width;
    m_WindowSizeH = height;
}

/* ----------------------------------------- */
/* --- setFrameRate() */
/**
* dăt[XLbvKvȏꍇǂU镑̎w
* PAKO_FRAME_SKIP_NONE = 0 t[XLbv
* PAKO_FRAME_SKIP_DISPLAY = 1 t[XLbvL(l)
*
* @param mode [in] t[XLbṽ[h
*
*/
void PakoSimpleApp::setFrameSkipMode(int mode)
{
    m_FrameSkipMode = mode;
}


/**
 * tXN[[h̐؂ւ\bh
 *
 * @param bfull [in] tXN[ɂ邩(true=full, false=window)
 */
void PakoSimpleApp::setFullScreen(bool bfull)
{
    if (bfull == true) {
        m_SdlVideoModeFlag |= SDL_FULLSCREEN;
    }
    else {
        m_SdlVideoModeFlag &= ~SDL_FULLSCREEN;
    }
    SDL_SetVideoMode(m_WindowSizeW,
        m_WindowSizeH,
        0,
        m_SdlVideoModeFlag);
}



/*------------------------------------------------------*/
/* private method                                       */
/*------------------------------------------------------*/

/* ----------------------------------------- */
/* --- tick() */
/**
* t[ĂяoQ[{̃\bh
* pŃI[o[ChĂɃQ[{̂Lq
* 1t[̍ƂI烁\bh𔲂邱
*
* @return trueŒʏƒAfalseŃQ[I
*
*/
bool PakoSimpleApp::tick(void)
{
    return true;
}


/* ----------------------------------------- */
/* --- tickDisplay() */
/**
* t[Ăяoʕ\\bh
* pŃI[o[ChĂɉʕ`揈Lq
* tick()̒ɋLqĂǂA̓t[XLbv
* ΂Ă΂Ȃ̂ŏ肭p
*
* @return trueŒʏƒAfalseŕ\ُ
*
*/
bool PakoSimpleApp::tickDisplay(void)
{
    return true;
}


/* ----------------------------------------- */
/* --- swapFrameBuffer() */
/**
* t[̍ŌŌĂяo郁\bh
* tickDisplay()̒ɌĂ΂AۂɃt[obt@
* ʂɕ\铮s
* p̃foCXɉĎ
*
*/
void PakoSimpleApp::swapFrameBuffer(void)
{
}

/*------------------------------------------------------*/
/* local method                                         */
/*------------------------------------------------------*/




/*---------------------------------------------------------------*/
/*                                                               */
/* Input class                                                   */
/*                                                               */
/*---------------------------------------------------------------*/

/* constructer and inisialize */
PakoSimpleApp::Input::Input(void)
{
    int i;
    int JoyNum;

    for(i=0; i<JOY_NUM_MAX; i++) {
        m_JoyPtr[i] = 0;
        m_JoyStickAxes[i] = 0;
        m_JoyStickButtons[i] = 0;
        m_JoyKey[i] = 0;
        m_JoyStick[i] = 0;
    }
    /* ----- WCXeBbN̑݊mF */
    JoyNum = SDL_NumJoysticks();
    if (JoyNum > JOY_NUM_MAX) {
        JoyNum = JOY_NUM_MAX;
    }
    for(i=0; i<(JoyNum + 1); i++) {
        m_JoyPtr[i] = SDL_JoystickOpen(i);
        if (m_JoyPtr[i] != NULL) {
            /* --- WCXeBbN̎ނ`FbN */
            m_JoyStickAxes[i] = SDL_JoystickNumAxes(m_JoyPtr[i]);
            m_JoyStickButtons[i] = SDL_JoystickNumButtons(m_JoyPtr[i]);
        }
    }
}

/* destructor */
PakoSimpleApp::Input::~Input(void)
{
    int i;

    /* ----- WCXeBbN̉ */
    for(i=0; i<JOY_NUM_MAX; i++) {
        if (m_JoyPtr[i] != 0) {
            SDL_JoystickClose(m_JoyPtr[i]);
        }
    }
}

/* ------------------------------------- */
/* --- accessor */

/**
 * L[͂̏擾
 * ߂l͉ꂽ{^̏Wrbg
 *
 * return ̓{^ԁA bit = 1
 */
unsigned int PakoSimpleApp::Input::getInput(int side)
{
    unsigned int r1, r2;

    r1 = r2 = 0;
    if (side < JOY_NUM_MAX) {
        r1 = m_JoyKey[side];
        r2 = m_JoyStick[side];
        r1 |= r2;
    }

    return(r1);
}

/**
 * L[͂̏擾AgK̂
 * ߂l͉ꂽ{^̏Wrbg
 *
 * return ̓{^ԁAꂽuԂ̂ bit = 1
 */
unsigned int PakoSimpleApp::Input::getInputTriger(int side)
{
    unsigned int  r1, r2;

    r1 = r2 = 0;
    if (side < JOY_NUM_MAX) {
        r1 = (m_JoyKey[side] ^ m_JoyKeyPast[side]) & m_JoyKey[side];
        r2 = (m_JoyStick[side] ^ m_JoyStickPast[side]) & m_JoyStick[side];
        r1 |= r2;
    }
    return(r1);
}


/* ------------------------------------- */
/* --- work method */
/**
 * SDL̓Cxg󂯎
 * t[ǂŏ邱
 */
void PakoSimpleApp::Input::poll(void)
{
    int  i;
    m_bAppExit = false;
    m_bFullScreen = false;

    /* ----- for Triger */
    for(i=0; i<JOY_NUM_MAX; i++) {
        m_JoyKeyPast[i] = m_JoyKey[i];
        m_JoyStickPast[i] = m_JoyStick[i];
    }

    /* ----- Key Event Read */
    while(SDL_PollEvent(&m_event) != 0) {
        switch(m_event.type) {
        case SDL_KEYUP:
            key_read_up(&m_event.key);
            break;

        case SDL_KEYDOWN:
            key_read_down(&m_event.key);
            break;

        case SDL_JOYBUTTONUP:
            joy_read_button_up_pc(&m_event.jbutton);
            break;

        case SDL_JOYBUTTONDOWN:
            joy_read_button_down_pc(&m_event.jbutton);
            break;

        case SDL_JOYAXISMOTION:
            joy_read_stick(&m_event.jaxis);
            break;

        case SDL_QUIT:
            m_bAppExit = 1;
            break;
        }
    }
}


/* ------------------------------------- */
/* --- input local method */

/* L[{[h */
void  PakoSimpleApp::Input::key_read_down(SDL_KeyboardEvent *key)
{
    if (key->keysym.sym == SDLK_UP)
        m_JoyKey[0] = m_JoyKey[0] | IN_Up;
    if (key->keysym.sym == SDLK_DOWN)
        m_JoyKey[0] = m_JoyKey[0] | IN_Down;
    if (key->keysym.sym == SDLK_LEFT)
        m_JoyKey[0] = m_JoyKey[0] | IN_Left;
    if (key->keysym.sym == SDLK_RIGHT)
        m_JoyKey[0] = m_JoyKey[0] | IN_Right;
    if (key->keysym.sym == SDLK_z)
        m_JoyKey[0] = m_JoyKey[0] | IN_Button1;
    if (key->keysym.sym == SDLK_x)
        m_JoyKey[0] = m_JoyKey[0] | IN_Button2;
    if (key->keysym.sym == SDLK_c)
        m_JoyKey[0] = m_JoyKey[0] | IN_Button3;
    if (key->keysym.sym == SDLK_v)
        m_JoyKey[0] = m_JoyKey[0] | IN_Button4;
    if (key->keysym.sym == SDLK_SPACE)
        m_JoyKey[0] = m_JoyKey[0] | IN_Button1;

    if (key->keysym.sym == SDLK_q)
        m_bAppExit = 1;
    if (key->keysym.sym == SDLK_ESCAPE)
        m_bAppExit = 1;
    if (key->keysym.sym == SDLK_F11)
        m_bFullScreen = 1;
}

/* L[{[h */
void  PakoSimpleApp::Input::key_read_up(SDL_KeyboardEvent *key)
{
    if (key->keysym.sym == SDLK_UP)
        m_JoyKey[0] = m_JoyKey[0] & (~(IN_Up));
    if (key->keysym.sym == SDLK_DOWN)
        m_JoyKey[0] = m_JoyKey[0] & (~(IN_Down));
    if (key->keysym.sym == SDLK_LEFT)
        m_JoyKey[0] = m_JoyKey[0] & (~(IN_Left));
    if (key->keysym.sym == SDLK_RIGHT)
        m_JoyKey[0] = m_JoyKey[0] & (~(IN_Right));
    if (key->keysym.sym == SDLK_z)
        m_JoyKey[0] = m_JoyKey[0] & (~(IN_Button1));
    if (key->keysym.sym == SDLK_x)
        m_JoyKey[0] = m_JoyKey[0] & (~(IN_Button2));
    if (key->keysym.sym == SDLK_c)
        m_JoyKey[0] = m_JoyKey[0] & (~(IN_Button3));
    if (key->keysym.sym == SDLK_v)
        m_JoyKey[0] = m_JoyKey[0] & (~(IN_Button4));
    if (key->keysym.sym == SDLK_SPACE)
        m_JoyKey[0] = m_JoyKey[0] & (~(IN_Button1));
}

/* WCXeBbN */
void  PakoSimpleApp::Input::joy_read_stick(SDL_JoyAxisEvent *stick)
{
    if (stick->which < JOY_NUM_MAX) {
        switch(stick->axis) {
            /* ---  */
        case 0:
            m_JoyStick[stick->which] &= (~(IN_Left|IN_Right));
            if (stick->value < (-JOY_STICK_DITHER)) {
                m_JoyStick[stick->which] = m_JoyStick[stick->which] | IN_Left;
            }
            if (stick->value > JOY_STICK_DITHER) {
                m_JoyStick[stick->which] = m_JoyStick[stick->which] | IN_Right;
            }
            break;

            /* --- c */
        case 1:
            m_JoyStick[stick->which] &= (~(IN_Up|IN_Down));
            if (stick->value < (-JOY_STICK_DITHER)) {
                m_JoyStick[stick->which] = m_JoyStick[stick->which] | IN_Up;
            }
            if (stick->value > JOY_STICK_DITHER) {
                m_JoyStick[stick->which] = m_JoyStick[stick->which] | IN_Down;
            }
            break;
        }
    }
}

void  PakoSimpleApp::Input::joy_read_button_up_pc(SDL_JoyButtonEvent *btn)
{
  if (btn->which < JOY_NUM_MAX) {
    if ((btn->button == 0) && (btn->state == SDL_RELEASED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] & (~(IN_Button1));
    if ((btn->button == 1) && (btn->state == SDL_RELEASED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] & (~(IN_Button2));
    if ((btn->button == 2) && (btn->state == SDL_RELEASED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] & (~(IN_Button3));
    if ((btn->button == 3) && (btn->state == SDL_RELEASED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] & (~(IN_Button4));
    if ((btn->button == 4) && (btn->state == SDL_RELEASED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] & (~(IN_Button5));
    if ((btn->button == 5) && (btn->state == SDL_RELEASED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] & (~(IN_Button6));
    if ((btn->button == 6) && (btn->state == SDL_RELEASED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] & (~(IN_Button7));
    if ((btn->button == 7) && (btn->state == SDL_RELEASED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] & (~(IN_Button8));
  }
}

void  PakoSimpleApp::Input::joy_read_button_down_pc(SDL_JoyButtonEvent *btn)
{
  if (btn->which < JOY_NUM_MAX) {
    if ((btn->button == 0) && (btn->state == SDL_PRESSED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] | IN_Button1;
    if ((btn->button == 1) && (btn->state == SDL_PRESSED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] | IN_Button2;
    if ((btn->button == 2) && (btn->state == SDL_PRESSED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] | IN_Button3;
    if ((btn->button == 3) && (btn->state == SDL_PRESSED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] | IN_Button4;
    if ((btn->button == 4) && (btn->state == SDL_PRESSED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] | IN_Button5;
    if ((btn->button == 5) && (btn->state == SDL_PRESSED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] | IN_Button6;
    if ((btn->button == 6) && (btn->state == SDL_PRESSED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] | IN_Button7;
    if ((btn->button == 7) && (btn->state == SDL_PRESSED))
      m_JoyKey[btn->which] = m_JoyKey[btn->which] | IN_Button8;
  }
}




/*---------------------------------------------------------------*/
/*                                                               */
/* Sound class                                                   */
/*                                                               */
/*---------------------------------------------------------------*/

/* constructer and inisialize */
PakoSimpleApp::Sound::Sound(int freq, int buf)
    : m_bEnable(false)
    , m_iSeTrack(0)
    , m_pSoundTrack(0)
{
    int  i;
    int  ret;

    std::vector<std::string> fileList;

    m_mapBgmList.clear();
    m_mapSePool.clear();

    /* ----- SDL_mixer ̃I[v */
    m_bEnable = true;
    ret = Mix_OpenAudio(freq, AUDIO_S16, 2, buf);
    if (ret != 0) {
        m_bEnable = false;
        return;
    }
    for(i=0; i<8; i++) {
        Mix_Volume(i, MIX_MAX_VOLUME);
    }
    Mix_VolumeMusic(MIX_MAX_VOLUME);

    /* ----- TEhXgǂݍ */
    {
        std::fstream fsRead(SOUND_LIST_FILE, std::ios::in );
        std::string  s1, s2;
        if (fsRead.fail() ) {
            m_bEnable = false;
            return;
        }
        while( !fsRead.eof() ) {
            int  header;
            fsRead >> s1 >> s2;
            header = s1.find('M');
            if (header != std::string::npos) {
                std::string number;
                number = s1.substr(1, s1.size()-1);
                m_mapBgmList.insert( std::map<int, std::string>::value_type( strtol(number.c_str(), NULL, 10), s2 ));
            }
            header = s1.find('S');
            if (header != std::string::npos) {
                std::string number;
                number = s1.substr(1, s1.size()-1);
                m_mapSePool.insert( std::map<int, Mix_Chunk*>::value_type( strtol(number.c_str(), NULL, 10), Mix_LoadWAV(s2.c_str()) ));
            }
        }
    }
}

PakoSimpleApp::Sound::~Sound(void)
{
  if (m_bEnable == false) {
    return;
  }
  /* ----- ێTEh̉ */
  if (m_pSoundTrack != 0) {
    Mix_HaltMusic();
    Mix_FreeMusic(m_pSoundTrack);
    m_pSoundTrack = 0;
  }
  /* ----- SE`l̒~ */
  for(int i=0; i<8; i++) {
    Mix_HaltChannel(i);
  }
  /* ----- SE chank ̉ */
  std::map<int, Mix_Chunk*>::iterator it;
  for(it=m_mapSePool.begin(); it!=m_mapSePool.end(); it++) {
      Mix_FreeChunk(it->second);
  }
  /* ----- SDL_Mixer ̃N[Y */
  Mix_CloseAudio();
}


void  PakoSimpleApp::Sound::StartMusic(int req, int loop)
{
    if (m_bEnable == false) {
        return;
    }
    /* --- ͈̓`FbN */
    if (m_mapBgmList.find(req) == m_mapBgmList.end()) {
        return;
    }
    /* ----- ܂ł̉t~ */
    if (m_pSoundTrack != 0) {
        Mix_HaltMusic();
        Mix_FreeMusic(m_pSoundTrack);
        m_pSoundTrack = 0;
    }
    /* ----- BGM tJn */
    m_pSoundTrack = Mix_LoadMUS(m_mapBgmList[req].c_str());
    if (m_pSoundTrack == NULL) {
        m_pSoundTrack = 0;
        return;
    }
    Mix_PlayMusic(m_pSoundTrack, loop);
}

void  PakoSimpleApp::Sound::Music(int req)
{
    StartMusic(req, -1);
}

void  PakoSimpleApp::Sound::MusicOneshot(int req)
{
    StartMusic(req, 1);
}


void  PakoSimpleApp::Sound::SE(int req)
{
    if (m_bEnable == false) {
        return;
    }
    /* --- ͈̓`FbN */
    if (m_mapSePool.find(req) == m_mapSePool.end()) {
        return;
    }
    /* ----- SE  */
    Mix_PlayChannel(m_iSeTrack, m_mapSePool[req], 0);
    m_iSeTrack = (m_iSeTrack + 1) % 8;
}

void  PakoSimpleApp::Sound::MusicStop(void)
{
    if (m_bEnable == false) {
        return;
    }
    /* --- */
    Mix_HaltMusic();
	if (m_pSoundTrack) {
		Mix_FreeMusic(m_pSoundTrack);
		m_pSoundTrack = 0;
	}
}

void  PakoSimpleApp::Sound::SEStop(void)
{
    if (m_bEnable == false) {
        return;
    }
    /* ----- SE`l̒~ */
    for(int i=0; i<8; i++) {
        Mix_HaltChannel(i);
    }
}

void  PakoSimpleApp::Sound::Volume(int value)
{
    if (m_bEnable == false) {
        return;
    }
    if (value > MIX_MAX_VOLUME) {
        value = MIX_MAX_VOLUME;
    }
    if (value < 0) {
        value = 0;
    }
    for(int i=0; i<8; i++) {
        Mix_Volume(i, value);
    }
    Mix_VolumeMusic(value);
}

 