/* ---------------------------------------------------------- */
/*  pako_simple_app.h                                         */
/* ---------------------------------------------------------- */

/*------------------------------------------------------------- */
/** @file
@brief    Pako SDL Game Library - Foundamental SDL application class
@author   rerofumi
@since    Nov.04.2006
*/
/*-----------------------------------------------------
Copyright (C) 2006,2011 fumi2kick/rerofumi <rero2@yuumu.org>
All Rights Reserved.
------------------------------------------------------
Update:
Apr.07.2011 - marge input/sound module
------------------------------------------------------*/

#ifndef PAKO_SIMPLE_APP_H
#define PAKO_SIMPLE_APP_H

/*-------------------------------*/
/* include                       */
/*-------------------------------*/

#include <string>
#include <map>

#include "SDL.h"
#include "SDL_main.h"
#include "SDL_Audio.h"
#include "SDL_mixer.h"

/*-------------------------------*/
/* define                        */
/*-------------------------------*/

//! t[XLbv
#define  PAKO_FRAME_SKIP_NONE  0
//! t[XLbvL
#define  PAKO_FRAME_SKIP_DISPLAY  1

//! ftHgʃTCYA
#define  PAKO_FRAME_SIZE_W  480
//! ftHgʃTCYAc
#define  PAKO_FRAME_SIZE_H  272

//! ftHgt[[g
#define  PAKO_FRAME_RATE  60

/*-------------------------------*/
/* struct                        */
/*-------------------------------*/

/*-------------------------------*/
/* class                         */
/*-------------------------------*/


/**
* @brief Q[AvP[V{NX
*        SDLx[X̃Q[AvP[VǗs
*        Q[{̂̌pNX
*
* @todo -
* @bug -
*/
class PakoSimpleApp
{

    /* ------------------------------------ */
    /* --- NX I/O NX */
    /* ------------------------------------ */
public:
    /* ---  */
    class Input {
    public:
        enum {
            IN_Up       = 0x0001,
            IN_Down     = 0x0002,
            IN_Left     = 0x0004,
            IN_Right    = 0x0008,
            IN_Button1  = 0x0010,
            IN_Button2  = 0x0020,
            IN_Button3  = 0x0040,
            IN_Button4  = 0x0080,
            IN_Button5  = 0x0100,
            IN_Button6  = 0x0200,
            IN_Button7  = 0x0400,
            IN_Button8  = 0x0800,
            IN_Button9  = 0x1000,
            IN_Button10 = 0x2000,
            IN_Button11 = 0x4000,
            IN_Button12 = 0x8000
        } InputBit;

        enum {
            JOY_NUM_MAX = 2,
            JOY_STICK_DITHER = 20000
        } InputDefine;

        /* constructer, destructor */
        Input(void);
        virtual ~Input(void);

        /* method */
        void  poll(void);
        bool  isExit(void) { return m_bAppExit; };
        bool  isFull(void) { return m_bFullScreen; };
        unsigned int   getInput(int side);
        unsigned int   getInputTriger(int side);

    protected:
        /* --- member */
        SDL_Event  m_event;
        SDL_Joystick  *m_JoyPtr[JOY_NUM_MAX];

        unsigned long  m_JoyKey[JOY_NUM_MAX];
        unsigned long  m_JoyStick[JOY_NUM_MAX];
        unsigned long  m_JoyKeyPast[JOY_NUM_MAX];
        unsigned long  m_JoyStickPast[JOY_NUM_MAX];
        /* WCXeBbN̏(ʏ̓WCpbhւƕϊ) */
        int m_JoyStickAxes[JOY_NUM_MAX];
        int m_JoyStickButtons[JOY_NUM_MAX];
        /* IL[tO */
        bool  m_bAppExit;
        /* XN[[h؂ւL[tO */
        bool  m_bFullScreen;

        /* --- local method */
        void  key_read_down(SDL_KeyboardEvent *key);
        void  key_read_up(SDL_KeyboardEvent *key);
        void  joy_read_stick(SDL_JoyAxisEvent *stick);
        void  joy_read_button_up_pc(SDL_JoyButtonEvent *btn);
        void  joy_read_button_down_pc(SDL_JoyButtonEvent *btn);
    };

    /* --- TEho */
    class Sound {
    public:
        /* constructer, destructor */
        Sound(int freq, int buf);
        virtual ~Sound(void);

        void  Music(int req);
        void  MusicOneshot(int req);
        void  SE(int req);
        void  MusicStop(void);
        void  SEStop(void);
        void  Volume(int value);
    protected:
        /* --- sound data map */
        std::map<int, std::string> m_mapBgmList;
        std::map<int, Mix_Chunk*> m_mapSePool;
        /* --- member */
        bool  m_bEnable;
        int   m_iSeTrack;
        Mix_Music  *m_pSoundTrack;

        void  StartMusic(int req, int loop);
    };


    /* ----------------------------------------------------*/
    /* --- NX{                                      */
    /* ----------------------------------------------------*/

    /* ----------------------------------- */
    /* --- RXgN^EfXgN^ */
public:
    PakoSimpleApp();
    virtual  ~PakoSimpleApp();

    /* ----------------------------------- */
    /* --- pubNo */
public:
    //! ^[Qbg@
    int m_TargetMachine;
    //! \TCY
    int m_WindowSizeW;
    //! \TCYc
    int m_WindowSizeH;
    //! video mode tO
    int m_SdlVideoModeFlag;
    //! t[[gݒ
    int m_FrameRate;
    //! t[XLbv[h
    int m_FrameSkipMode;
    //! t[XLbv
    int m_SkipRate;
    //! Ñt[JE^
    int m_AppCounter;
    //! tXN[[h
    bool m_bFullScreen;
    //! t[^C~O
    int  m_SystemTime;
    //! Ot[̎
    int  m_BeforeTiming;
	//! SDL_mixer ̋쓮g
	int m_iSoundParamFreq;
	//! SDL_mixer ̔`Nobt@TCY
	int m_iSoundParamBuff;

    /* ----------------------------------- */
    /* --- thNXo */
protected:
	//! EBhE^Cg
	std::string m_strWindowTitle;

    /* ----------------------------------- */
    /* --- vCx[go */
private:
    //! {^͊Ǘ̃CX^X
    Input  *m_pInput;
    //! TEhǗ̃CX^X
    Sound  *m_pSound;

    /* ----------------------------------- */
    /* --- pubN\bh */
public:
    int execute(void);
	void setWindowTitle(const char *name) { m_strWindowTitle = name; };
    void setFrameRate(int fps);
    void setFrameSkipMode(int mode);
    void setScreenSize(int width, int height);
    void setFullScreen(bool bfull);
    Input* getInput(void) { return m_pInput; }
    Sound* getSound(void) { return m_pSound; }

    /* ----------------------------------- */
    /* --- thNX\bh */
protected:
    virtual int onBoot(void);
    virtual int onShutdown(void);
    virtual int onExecuteBefore(void);
    virtual int onExecuteAfter(void);
    virtual bool tick(void);
    virtual bool tickDisplay(void);
    virtual void swapFrameBuffer(void);

    /* ----------------------------------- */
    /* --- vCx[g\bh */
private:



};


#endif

