/* ---------------------------------------------------------- */
/*  pako_state_machine.h                                      */
/* ---------------------------------------------------------- */

/*------------------------------------------------------------- */
/** @file
    @brief    Pako SDL Game Library - state machine & manage class
    @author   rerofumi
    @since    Apr.09.2011
*/
/*-----------------------------------------------------
 Copyright (C) 2011 fumi2kick/rerofumi <rero2@yuumu.org>
 All Rights Reserved.
 -----------------------------------

 ------------------------------------------------------*/

/*-------------------------------*/
/* include                       */
/*-------------------------------*/

#include "pako_state_machine.h"
#include "pako_simple_app.h"

/*-------------------------------*/
/* define                        */
/*-------------------------------*/

/*-------------------------------*/
/* global value                  */
/*-------------------------------*/

/*-------------------------------*/
/* local routine                 */
/*-------------------------------*/

/*-------------------------------*/
/* implementatiion               */
/*-------------------------------*/

/*---------------------------------------------------------------*/
/*                                                               */
/* Application base class                                        */
/*                                                               */
/*---------------------------------------------------------------*/

/*------------------------------------------------------*/
/* constructor/destructor                               */
/*------------------------------------------------------*/

/**
* RXgN^
*/
PakoStateMachine::PakoStateMachine(PakoSimpleApp *pApp, const std::string &strStateName)
    :m_pAppClass(pApp)
    ,m_strStateName(strStateName)
    ,m_bExecute(false)
    ,m_strNextState("")
    ,m_strSubState("")
{
    m_mapSubState.clear();
}

/**
* fXgN^(n)
*/
PakoStateMachine::~PakoStateMachine()
{
	freeSubState();
}


/*------------------------------------------------------*/
/* interface                                            */
/*------------------------------------------------------*/

/* ----------------------------------- */
/* --- tick() */
/**
 * t[̎sC
 * isZn[N
 */
//virtual
void PakoStateMachine::tick()
{
}

/* ----------------------------------- */
/* --- tickDisplay() */
/**
 * t[̎sC
 * ʕ\n[N
 */
//virtual
void PakoStateMachine::tickDisplay()
{
}


/* ----------------------------------- */
/* --- getName() */
/**
 * ̃Xe[gn
 *
 * @param strname [out] Xe[g
 */
void PakoStateMachine::getName(std::string &strName)
{
    strName = m_strStateName;
}


/* ----------------------------------- */
/* --- doReset() */
/**
 * ̃Xe[gŏs
 */
void PakoStateMachine::doReset()
{
	freeSubState();
    onBegin();
    m_strNextState = "";
    m_bExecute = true;
}

/* ----------------------------------- */
/* --- doStop() */
/**
 * f
 */
void PakoStateMachine::doStop()
{
    onEnd();
}


/* ----------------------------------- */
/* --- getApp() */
/**
 * AvP[VNX̃CX^Xn
 *
 * @return AvP[VNX̃CX^Xւ̃|C^
 */
PakoSimpleApp* PakoStateMachine::getApp()
{
    return m_pAppClass;
}

/* ----------------------------------- */
/* --- isDone() */
/**
 * ̃Xe[gIn
 *
 * @retval true Xe[g̎s͏IĂ܂
 * @retval false Xe[gs
 */
bool PakoStateMachine::isDone()
{
    return !m_bExecute;
}


/* ----------------------------------- */
/* --- getNextState() */
/**
 * Xe[gIAɈڍsXe[gʂɓn
 *
 * @param strNextStateName [out] ڍs̃Xe[g
 * @return Xe[gݒ肳Ă true / ڍsȂȂ false
 */
bool PakoStateMachine::getNextState(std::string &strNextStateName)
{
    strNextStateName = m_strNextState;

    return (m_strNextState.size() > 0);
}


/*------------------------------------------------------*/
/* local method                                         */
/*------------------------------------------------------*/

/**
 * Xe[gJn̏
 * Xe[g}l[W[̓TuXe[g̈ꗗȂǂs
 */
//virtual
void PakoStateMachine::onBegin(void)
{
}

/**
 * Xe[ǧn
 * ɈڍsXe[g̋LqȂǂōs
 */
//virtual
void PakoStateMachine::onEnd(void)
{
}


/*------------------------------------------------------*/
/* local method                                         */
/*------------------------------------------------------*/

//! Xe[gTuXe[g̃CX^X擾
PakoStateMachine* PakoStateMachine::getSubState(std::string &strName)
{
    PakoStateMachine *pt = 0;
    std::map<std::string, PakoStateMachine*>::iterator it;

    it = m_mapSubState.find(strName);
    if (it != m_mapSubState.end()) {
        pt = it->second;
    }

    return pt;
}

//! TuXe[gǗɒǉ
void PakoStateMachine::addSubState(PakoStateMachine *stm)
{
    std::string name;

    stm->getName( name );
    m_mapSubState.insert( std::map<std::string, PakoStateMachine*>::value_type( name, stm ) );
}

//! w薼̃TuXe[g}Vs
bool PakoStateMachine::beginSubState(std::string &strSubStateName)
{
    PakoStateMachine *pt = 0;

    pt = getSubState( strSubStateName );
    if (pt) {
        pt->doReset();
        m_strSubState = strSubStateName;
    }

    return (pt != 0);
}

//! ǗĂXe[g}VJăNA
void  PakoStateMachine::freeSubState(void)
{
    std::map<std::string, PakoStateMachine*>::iterator it;

    // TuXe[g̊J
    for(it=m_mapSubState.begin(); it!=m_mapSubState.end(); it++) {
        delete it->second;
        it->second = 0;
    }
    m_mapSubState.clear();
}


/*------------------------------------------------------*/
/* substate machine                                     */
/*------------------------------------------------------*/

/**
 * TuXe[g}V̎s
 * Xe[g}V}l[W[łƂAtick() 炱Ă
 */
bool PakoStateMachine::doSubState()
{
    bool result = true;
    PakoStateMachine *pt = 0;

    if (m_strSubState.size() == 0) {
        return false;
    }

    pt = getSubState( m_strSubState );
    if (pt) {
        pt->tick();
        if (true == pt->isDone()) {
            std::string next;
            if (pt->getNextState( next )) {
                pt = getSubState( next );
                if (pt) {
                    pt->doReset();
                    m_strSubState = next;
                }
                else {
                    /* ̃Xe[gȂ */
                    result = false;
                    m_strSubState = "";
                }
            }
            else {
                /* ̃TuXe[gw肳ĂȂ̂ŏI */
                result = false;
                m_strSubState = "";
            }
        }
    }

    return result;
}

/**
 * TuXe[g}V\n̎s
 * Xe[g}V}l[W[łƂAtickDisplay() 炱Ă
 */
void PakoStateMachine::doSubStateDisplay()
{
    PakoStateMachine *pt = 0;

    pt = getSubState( m_strSubState );
    if (pt) {
        pt->tickDisplay();
    }
}



